@@ -1,11 +1,12 @@
 use std::time::Duration;
+use serde::{Serialize, Deserialize};
 
 use crate::{
-    ComponentDescription, ComponentGeneratorId, ComponentLocation, ComponentPath, ComponentType,
+    ComponentDescription, ComponentId, ComponentLocation, ComponentType,
     ConcurrencyMode, Delegate, SuiteAttributes,
 };
 
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct BookEndAttributes {
     /// Indicates that bookend should not be run.
     pub ignore: bool,
@@ -64,11 +65,10 @@ impl<TParameters> BookEnd<TParameters> {
     pub fn new_setup(
         parent_suite_description: &ComponentDescription,
         parent_suite_attributes: &SuiteAttributes,
-        id_gen: &mut ComponentGeneratorId,
+        id: ComponentId,
         name: Option<&'static str>,
         description: Option<&'static str>,
-        path: &'static str,
-        src: Option<ComponentLocation>,
+        location: ComponentLocation,
         ignore: Option<bool>,
         time_limit: Option<Duration>,
         concurrency_mode: Option<ConcurrencyMode>,
@@ -76,14 +76,13 @@ impl<TParameters> BookEnd<TParameters> {
     ) -> Self {
         Self {
             description: ComponentDescription::new(
-                ComponentPath::from(path),
                 name,
-                id_gen.next(),
-                parent_suite_description.path().clone(),
+                id,
                 parent_suite_description.id().clone(),
+                location,
+                parent_suite_description.location().clone(),
                 description,
                 ComponentType::Setup,
-                src,
             ),
             attributes: BookEndAttributes::new_setup(
                 parent_suite_attributes,
@@ -98,11 +97,10 @@ impl<TParameters> BookEnd<TParameters> {
     pub fn new_tear_down(
         parent_suite_description: &ComponentDescription,
         parent_suite_attributes: &SuiteAttributes,
-        id_gen: &mut ComponentGeneratorId,
+        id: ComponentId,
         name: Option<&'static str>,
         description: Option<&'static str>,
-        path: &'static str,
-        src: Option<ComponentLocation>,
+        location: ComponentLocation,
         ignore: Option<bool>,
         time_limit: Option<Duration>,
         concurrency_mode: Option<ConcurrencyMode>,
@@ -110,14 +108,13 @@ impl<TParameters> BookEnd<TParameters> {
     ) -> Self {
         Self {
             description: ComponentDescription::new(
-                ComponentPath::from(path),
                 name,
-                id_gen.next(),
-                parent_suite_description.path().clone(),
+                id,
                 parent_suite_description.id().clone(),
+                location,
+                parent_suite_description.location().clone(),
                 description,
                 ComponentType::TearDown,
-                src,
             ),
             attributes: BookEndAttributes::new_tear_down(
                 parent_suite_attributes,
@@ -128,4 +125,4 @@ impl<TParameters> BookEnd<TParameters> {
             bookend_fn: setup_fn,
         }
     }
-}
+}
\ No newline at end of file
