@@ -2,9 +2,10 @@
 macro_rules! src_loc {
     () => {
         $crate::ComponentLocation {
-            file_name: file!(),
+            file_name: std::borrow::Cow::from(file!()),
             column: column!(),
             line: line!(),
+            path: $crate::ComponentPath::from(module_path!())
         }
     };
 }
