@@ -1,11 +1,12 @@
 use std::time::Duration;
+use serde::{Serialize, Deserialize};
 
 use crate::{
-    ComponentDescription, ComponentGeneratorId, ComponentLocation, ComponentPath, ComponentType,
+    ComponentDescription, ComponentId, ComponentLocation, ComponentType,
     ConcurrencyMode, Delegate, SuiteAttributes, TestParameters,
 };
 
-#[derive(Clone, Debug, PartialEq, Eq, Hash)]
+#[derive(Clone, Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub struct TestAttributes {
     /// Indicates that test should be run, however failures should be ignored and do not cascade.
     pub allow_fail: bool,
@@ -70,11 +71,10 @@ impl<TParameters: TestParameters> Test<TParameters> {
         parent_description: &ComponentDescription,
         parent_attributes: &SuiteAttributes,
         parameters: &TParameters,
-        id_gen: &mut ComponentGeneratorId,
+        id: ComponentId,
         name: Option<&'static str>,
         description: Option<&'static str>,
-        path: &'static str,
-        src: Option<ComponentLocation>,
+        location: ComponentLocation,
         ignore: Option<bool>,
         allow_fail: Option<bool>,
         warning_time_limit: Option<Duration>,
@@ -84,14 +84,13 @@ impl<TParameters: TestParameters> Test<TParameters> {
     ) -> Self {
         Self {
             description: ComponentDescription::new(
-                ComponentPath::from(path),
                 name,
-                id_gen.next(),
-                parent_description.path().clone(),
+                id,
                 parent_description.id().clone(),
+                location,
+                parent_description.location().clone(),
                 description,
                 ComponentType::Test,
-                src,
             ),
             attributes: TestAttributes::new(
                 parent_attributes,
